import datetime
import unittest

from elasticsearch_partition import formatters, partitioning


class TestTimeWindow(unittest.TestCase):

    def setUp(self):
        self.tm = partitioning.TimeWindow
        self.since = datetime.date(2018, 5, 8)
        self.until = datetime.date(2018, 6, 10)

    def test_init(self):
        # Test with wrong since type
        with self.assertRaises(TypeError):
            self.tm("42", self.until)

        # Test with wrong until type
        with self.assertRaises(TypeError):
            self.tm(self.since, "42")

        # Test with since more than until
        with self.assertRaises(ValueError):
            self.tm(self.until, self.since)

        # Test succes initialization
        instance = self.tm(self.since, self.until)
        self.assertEqual(instance.since, self.since)
        self.assertEqual(instance.until, self.until)

    def test_delta_years(self):
        instance = self.tm(
            datetime.date(2015, 5, 8),
            datetime.date(2018, 6, 10)
        )
        self.assertEqual(instance.delta_years, 3)

    def test_delta_days(self):
        instance = self.tm(self.since, self.until)
        expected = (self.until - self.since).days
        self.assertEqual(instance.delta_days, expected)

    def test_iteration(self):
        instance = self.tm(self.since, self.until)
        actual = len(list(instance))
        expected = (self.until - self.since).days + 1
        self.assertEqual(actual, expected)


class TestRangePartitioning(unittest.TestCase):

    def setUp(self):
        self.cls = partitioning.RangePartitioning
        self.since = datetime.date(2014, 9, 27)
        self.until = datetime.date(2018, 2, 4)

    def test_init(self):
        # Test with wrong formatter class
        with self.assertRaises(TypeError):
            self.cls(formatter=list)

    def test_init_default(self):
        instance = self.cls()
        self.assertEqual(instance._frequency, partitioning.DAY)
        self.assertIsInstance(
            instance._formatter,
            formatters.BigEndianDateFormatter
        )
        self.assertEqual(instance._escape, "*")

    def test_init_custom(self):

        def test_now():
            pass

        instance = self.cls(
            frequency=partitioning.MONTH,
            formatter=formatters.LittleEndianDateFormatter(),
            escape="@",
            now_func=test_now
        )
        self.assertEqual(instance._frequency, partitioning.MONTH)
        self.assertIsInstance(
            instance._formatter,
            formatters.LittleEndianDateFormatter
        )
        self.assertEqual(instance._escape, "@")
        self.assertEqual(instance._now_func, test_now)

    def test_partition_by_day(self):
        partition = self.cls(frequency=partitioning.DAY)
        expected = [
            "logs-2014-09-27",
            "logs-2014-09-28",
            "logs-2014-09-29",
            "logs-2014-09-30",
            "logs-2014-10-*",
            "logs-2014-11-*",
            "logs-2014-12-*",
            "logs-2015-*",
            "logs-2016-*",
            "logs-2017-*",
            "logs-2018-01-*",
            "logs-2018-02-01",
            "logs-2018-02-02",
            "logs-2018-02-03",
            "logs-2018-02-04",
        ]
        actual = partition("logs-*", self.since, self.until)
        self.assertListEqual(actual, expected)

    def test_partition_by_day_only_since(self):
        partition = self.cls(frequency=partitioning.DAY)
        partition._now_func = lambda: datetime.date(2018, 7, 4)
        expected = [
            "logs-2014-09-27",
            "logs-2014-09-28",
            "logs-2014-09-29",
            "logs-2014-09-30",
            "logs-2014-10-*",
            "logs-2014-11-*",
            "logs-2014-12-*",
            "logs-2015-*",
            "logs-2016-*",
            "logs-2017-*",
            "logs-2018-01-*",
            "logs-2018-02-*",
            "logs-2018-03-*",
            "logs-2018-04-*",
            "logs-2018-05-*",
            "logs-2018-06-*",
            "logs-2018-07-01",
            "logs-2018-07-02",
            "logs-2018-07-03",
            "logs-2018-07-04",
        ]
        actual = partition("logs-*", self.since)
        self.assertSequenceEqual(actual, expected)

    def test_partition_by_day_only_until(self):
        partition = self.cls(frequency=partitioning.DAY)
        partition._now_func = lambda: datetime.date(2018, 7, 4)
        expected = [
            "-logs-2018-02-04",
            "-logs-2018-02-05",
            "-logs-2018-02-06",
            "-logs-2018-02-07",
            "-logs-2018-02-08",
            "-logs-2018-02-09",
            "-logs-2018-02-10",
            "-logs-2018-02-11",
            "-logs-2018-02-12",
            "-logs-2018-02-13",
            "-logs-2018-02-14",
            "-logs-2018-02-15",
            "-logs-2018-02-16",
            "-logs-2018-02-17",
            "-logs-2018-02-18",
            "-logs-2018-02-19",
            "-logs-2018-02-20",
            "-logs-2018-02-21",
            "-logs-2018-02-22",
            "-logs-2018-02-23",
            "-logs-2018-02-24",
            "-logs-2018-02-25",
            "-logs-2018-02-26",
            "-logs-2018-02-27",
            "-logs-2018-02-28",
            "-logs-2018-03-*",
            "-logs-2018-04-*",
            "-logs-2018-05-*",
            "-logs-2018-06-*",
            "-logs-2018-07-01",
            "-logs-2018-07-02",
            "-logs-2018-07-03",
            "-logs-2018-07-04",
            "logs-*"
        ]
        actual = partition("logs-*", until=self.until)
        self.assertSequenceEqual(actual, expected)

    def test_partition_by_month(self):
        partition = self.cls(frequency=partitioning.MONTH)
        expected = [
            "logs-2014-09",
            "logs-2014-10",
            "logs-2014-11",
            "logs-2014-12",
            "logs-2015-*",
            "logs-2016-*",
            "logs-2017-*",
            "logs-2018-01",
            "logs-2018-02",
        ]
        actual = partition("logs-*", self.since, self.until)
        self.assertListEqual(actual, expected)

    def test_partition_by_month_only_since(self):
        partition = self.cls(frequency=partitioning.MONTH)
        partition._now_func = lambda: datetime.date(2018, 7, 4)
        expected = [
            "logs-2014-09",
            "logs-2014-10",
            "logs-2014-11",
            "logs-2014-12",
            "logs-2015-*",
            "logs-2016-*",
            "logs-2017-*",
            "logs-2018-01",
            "logs-2018-02",
            "logs-2018-03",
            "logs-2018-04",
            "logs-2018-05",
            "logs-2018-06",
            "logs-2018-07",
        ]
        actual = partition("logs-*", self.since)
        self.assertListEqual(actual, expected)

    def test_partition_by_month_only_until(self):
        partition = self.cls(frequency=partitioning.MONTH)
        partition._now_func = lambda: datetime.date(2018, 7, 4)
        expected = [
            "-logs-2018-02",
            "-logs-2018-03",
            "-logs-2018-04",
            "-logs-2018-05",
            "-logs-2018-06",
            "-logs-2018-07",
            "logs-*"
        ]
        actual = partition("logs-*", until=self.until)
        self.assertListEqual(actual, expected)

    def test_partition_by_year(self):
        partition = self.cls(frequency=partitioning.YEAR)
        expected = [
            "logs-2014",
            "logs-2015",
            "logs-2016",
            "logs-2017",
            "logs-2018",
        ]
        actual = partition("logs-*", self.since, self.until)
        self.assertListEqual(actual, expected)

    def test_partition_by_year_only_since(self):
        partition = self.cls(frequency=partitioning.YEAR)
        partition._now_func = lambda: datetime.date(2018, 7, 4)
        expected = [
            "logs-2014",
            "logs-2015",
            "logs-2016",
            "logs-2017",
            "logs-2018",
        ]
        actual = partition("logs-*", self.since)
        self.assertListEqual(actual, expected)

    def test_partition_by_year_only_until(self):
        partition = self.cls(frequency=partitioning.YEAR)
        partition._now_func = lambda: datetime.date(2018, 7, 4)
        expected = ["logs-*"]
        actual = partition("logs-*", until=self.until)
        self.assertListEqual(actual, expected)



from abc import ABCMeta, abstractmethod
import datetime


INVALID_SEP_CHARS = ["\\", "/", "*", "?", "\"", "<", ">", "|", " ", ","]

# Abstract metaclass with py2/py3 support
Abstract = ABCMeta("Abstract", (object,), {})


class DateFormatter(Abstract):
    """
    Abstract date formatter class.
    """

    def __init__(self, sep="-"):
        """
        Accepts separation character and initialize specified
        date formatter.
        """
        if sep in INVALID_SEP_CHARS:
            raise ValueError(
                "The separation character '%s' is not valid" % sep
            )
        self._sep = sep

    @abstractmethod
    def fmt_year(self, year, wildcard=False):
        pass

    @abstractmethod
    def fmt_month(self, year, month, wildcard=False):
        pass

    @abstractmethod
    def fmt_day(self, year, month, day):
        pass


class BigEndianDateFormatter(DateFormatter):
    """
    In this format the most significant data item is written before
    lesser data items i.e. year before month before day.

    (year, month, day), e.g. 2018-04-22 or 2018.04.22 or 2018/04/22
    """

    def fmt_year(self, year, wildcard=False):
        """
        Format and returns year in big-endian order style.

        :param int year: Year
        :param bool wildcard: Use wildcard instead day and month
        :rtype: str
        """
        date = datetime.date(year=year, month=1, day=1)
        if wildcard:
            return date.strftime("%Y{sep}*".format(sep=self._sep))
        return date.strftime("%Y")

    def fmt_month(self, year, month, wildcard=False):
        """
        Format and returns month in big-endian order style.

        :param int year: Year
        :param int month: Month
        :param bool wildcard: Use wildcard instead day
        :rtype: str
        """
        date = datetime.date(year=year, month=month, day=1)
        if wildcard:
            return date.strftime("%Y{sep}%m{sep}*".format(sep=self._sep))
        return date.strftime("%Y{sep}%m".format(sep=self._sep))

    def fmt_day(self, year, month, day):
        """
        Format and returns day in big-endian order style.

        :param int year: Year
        :param int month: Month
        :param int day: Day
        :rtype: str
        """
        date = datetime.date(year, month, day)
        return date.strftime("%Y{sep}%m{sep}%d".format(sep=self._sep))


class LittleEndianDateFormatter(DateFormatter):
    """
    In this format the most significant data item is written after
    lesser data items i.e. day before month before year.

    (day, month, year), e.g. 22-04-2018 or 22.04.2018 or 22/04/2018
    """

    def fmt_year(self, year, wildcard=False):
        """
        Format and returns year in little-endian order style.

        :param int year: Year
        :param bool wildcard: Use wildcard instead day and month
        :rtype: str
        """
        date = datetime.date(year=year, month=1, day=1)
        if wildcard:
            return date.strftime("*{sep}%Y".format(sep=self._sep))
        return date.strftime("%Y".format(sep=self._sep))

    def fmt_month(self, year, month, wildcard=False):
        """
        Format and returns month in little-endian order style.

        :param int year: Year
        :param int month: Month
        :param bool wildcard: Use wildcard instead day
        :rtype: str
        """
        date = datetime.date(year=year, month=month, day=1)
        if wildcard:
            return date.strftime("*{sep}%m{sep}%Y".format(sep=self._sep))
        return date.strftime("%m{sep}%Y".format(sep=self._sep))

    def fmt_day(self, year, month, day):
        """
        Format and returns day in little-endian order style.

        :param int year: Year
        :param int month: Month
        :param int day: Day
        :rtype: str
        """
        date = datetime.date(year, month, day)
        return date.strftime("%d{sep}%m{sep}%Y".format(sep=self._sep))


class MiddleEndianDateFormatter(DateFormatter):
    """
    In this format the month data item is written before day before year.

    (month, day, year), e.g. 04-22-2018 or 04.22.2018 or 04/22/2018
    """

    def fmt_year(self, year, wildcard=False):
        """
        Format and returns year in middle-endian order style.

        :param int year: Year
        :param bool wildcard: Use wildcard instead day and month
        :rtype: str
        """
        date = datetime.date(year=year, month=1, day=1)
        if wildcard:
            return date.strftime("*{sep}%Y".format(sep=self._sep))
        return date.strftime("%Y")

    def fmt_month(self, year, month, wildcard=False):
        """
        Format and returns month in middle-endian order style.

        :param int year: Year
        :param int month: Month
        :param bool wildcard: Use wildcard instead day
        :rtype: str
        """
        date = datetime.date(year=year, month=month, day=1)
        if wildcard:
            return date.strftime("%m{sep}*{sep}%Y".format(sep=self._sep))
        return date.strftime("%m{sep}%Y".format(sep=self._sep))

    def fmt_day(self, year, month, day):
        """
        Format and returns day in middle-endian order style.

        :param int year: Year
        :param int month: Month
        :param int day: Day
        :rtype: str
        """
        date = datetime.date(year, month, day)
        return date.strftime("%m{sep}%d{sep}%Y".format(sep=self._sep))



import unittest

from elasticsearch_partition import formatters


class TestBigEndianDateFormatter(unittest.TestCase):

    def setUp(self):
        self.module = formatters.BigEndianDateFormatter
        self.formatter = self.module()

    def test_fmt_year(self):
        # Test without wildcard
        actual = self.formatter.fmt_year(2018)
        self.assertEqual(actual, "2018")
        # Test with wildcard
        actual = self.formatter.fmt_year(2018, wildcard=True)
        self.assertEqual(actual, "2018-*")

    def test_fmt_month(self):
        # Test without wildcard
        actual = self.formatter.fmt_month(2018, 4)
        self.assertEqual(actual, "2018-04")
        # Test with wildcard
        actual = self.formatter.fmt_month(2018, 4, wildcard=True)
        self.assertEqual(actual, "2018-04-*")

    def test_fmt_day(self):
        actual = self.formatter.fmt_day(2018, 4, 20)
        self.assertEqual(actual, "2018-04-20")

    def test_sep(self):
        formatter = self.module(sep=".")
        actual_year = formatter.fmt_year(2018)
        actual_month = formatter.fmt_month(2018, 4)
        actual_day = formatter.fmt_day(2018, 4, 20)
        self.assertEqual(actual_year, "2018")
        self.assertEqual(actual_month, "2018.04")
        self.assertEqual(actual_day, "2018.04.20")

        for sep in formatters.INVALID_SEP_CHARS:
            with self.assertRaises(ValueError):
                self.module(sep)


class TestLittleEndianDateFormatter(unittest.TestCase):

    def setUp(self):
        self.module = formatters.LittleEndianDateFormatter
        self.formatter = self.module()

    def test_fmt_year(self):
        # Test without wildcard
        actual = self.formatter.fmt_year(2018)
        self.assertEqual(actual, "2018")
        # Test with wildcard
        actual = self.formatter.fmt_year(2018, wildcard=True)
        self.assertEqual(actual, "*-2018")

    def test_fmt_month(self):
        # Test without wildcard
        actual = self.formatter.fmt_month(2018, 4)
        self.assertEqual(actual, "04-2018")
        # Test with wildcard
        actual = self.formatter.fmt_month(2018, 4, wildcard=True)
        self.assertEqual(actual, "*-04-2018")

    def test_fmt_day(self):
        actual = self.formatter.fmt_day(2018, 4, 20)
        self.assertEqual(actual, "20-04-2018")

    def test_sep(self):
        formatter = self.module(sep=".")
        actual_year = formatter.fmt_year(2018)
        actual_month = formatter.fmt_month(2018, 4)
        actual_day = formatter.fmt_day(2018, 4, 20)
        self.assertEqual(actual_year, "2018")
        self.assertEqual(actual_month, "04.2018")
        self.assertEqual(actual_day, "20.04.2018")

        for sep in formatters.INVALID_SEP_CHARS:
            with self.assertRaises(ValueError):
                self.module(sep)


class TestMiddleEndianDateFormatter(unittest.TestCase):

    def setUp(self):
        self.module = formatters.MiddleEndianDateFormatter
        self.formatter = self.module()

    def test_fmt_year(self):
        # Test without wildcard
        actual = self.formatter.fmt_year(2018)
        self.assertEqual(actual, "2018")
        # Test with wildcard
        actual = self.formatter.fmt_year(2018, wildcard=True)
        self.assertEqual(actual, "*-2018")

    def test_fmt_month(self):
        # Test without wildcard
        actual = self.formatter.fmt_month(2018, 4)
        self.assertEqual(actual, "04-2018")
        # Test with wildcard
        actual = self.formatter.fmt_month(2018, 4, wildcard=True)
        self.assertEqual(actual, "04-*-2018")

    def test_fmt_day(self):
        actual = self.formatter.fmt_day(2018, 4, 20)
        self.assertEqual(actual, "04-20-2018")

    def test_sep(self):
        formatter = self.module(sep=".")
        actual_year = formatter.fmt_year(2018)
        actual_month = formatter.fmt_month(2018, 4)
        actual_day = formatter.fmt_day(2018, 4, 20)
        self.assertEqual(actual_year, "2018")
        self.assertEqual(actual_month, "04.2018")
        self.assertEqual(actual_day, "04.20.2018")

        for sep in formatters.INVALID_SEP_CHARS:
            with self.assertRaises(ValueError):
                self.module(sep)


if __name__ == "__main__":
    unittest.main()



import glob
from os import path
import re

import setuptools

try:
    from Cython.Distutils import build_ext
    CYTHON = True
except ImportError:
    CYTHON = False

BASE_DIR = path.abspath(path.dirname(__file__))


def find_version(fname):
    """Attempts to find the version number in the file names fname.
    Raises RuntimeError if not found.
    """
    version = ""
    with open(path.join(BASE_DIR, fname), "r") as fp:
        regex = re.compile(r'__version__ = [\'"]([^\'"]*)[\'"]')
        for line in fp:
            m = regex.match(line)
            if m:
                version = m.group(1)
                break
    if not version:
        raise RuntimeError("Cannot find version information")
    return version


def read(fname):
    with open(path.join(BASE_DIR, fname), "r") as fh:
        return fh.read()


if CYTHON:
    def modules(dirname):
        filenames = glob.glob(path.join(dirname, "*.py"))

        module_names = []
        for name in filenames:
            module, ext = path.splitext(path.basename(name))
            if module != "__init__":
                module_names.append(module)

        return module_names

    package_names = ["elasticsearch_partition"]
    ext_modules = [
        setuptools.Extension(
            package + "." + module,
            [path.join(*(package.split(".") + [module + ".py"]))]
        )
        for package in package_names
        for module in modules(path.join(BASE_DIR, *package.split(".")))
    ]

    cmdclass = {"build_ext": build_ext}
else:
    ext_modules = []
    cmdclass = {}


setuptools.setup(
    name="elasticsearch_partition",
    version=find_version("elasticsearch_partition/__init__.py"),
    author="Dmitri Vasilishin",
    author_email="vasilishin.d.o@gmail.com",
    description="A Python library for creating Elasticsearch partitioned "
                "indexes by date range",
    long_description=read("README.md"),
    url="https://github.com/kandziu/elasticsearch-partition",
    packages=setuptools.find_packages(exclude=("tests")),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords=["elasticsearch", "partition", "partitioning"],
    tests_require=["coverage"],
    extras_require={"dev": ["tox", "bumpversion"]},
    cmdclass=cmdclass,
    ext_modules=ext_modules
)



"""This module provide range partition method for your Elasticsearch
indexes. If you have a logic index partition you could use this module
for creating and searching through indexes by time range.

A partition is a division of a logical database or its constituent
elements into distinct independent parts. Database partitioning is
normally done for manageability, performance or availability reasons,
or for load balancing.
"""

from calendar import monthrange
import datetime
from itertools import groupby

from .formatters import BigEndianDateFormatter, DateFormatter


__all__ = ("RangePartitioning", "TimeWindow", "partition")


# Public constants for convenience
YEAR = 1
MONTH = 2
DAY = 3

_MAXMONTH = 12


def _group_by(iterable, key):
    """
    Group By implementation.

    :param Iterator iterable: Iterable sequesnce of date objects
    :param callable key: Function for getting attributes
    :yield (str, Iterator): Key with grouped values
    """
    for key, values in groupby(iterable, key=key):
        yield key, values


def _group_by_year(iterable):
    """
    Group and returns iterable sequence by year.

    :param Iterator iterable: Iterable sequesnce of date objects
    :return (year: str, Iterator): Grouped dates by year
    """
    return _group_by(iterable, key=lambda x: x.year)


def _group_by_month(iterable):
    """
    Group and returns iterable sequence by month.

    :param Iterator iterable: Iterable sequesnce of date objects
    :return (month: str, Iterator): Grouped dates by month
    """
    return _group_by(iterable, key=lambda x: x.month)


def _exclude(pattern):
    """
    Change index name to Elasticsearch exclude style.

    :param str pattern: Index name with a specified symbol
    :rtype: str
    """
    return "-" + pattern


class TimeWindow:
    __slots__ = ("since", "until")

    def __init__(self, since, until):
        if not isinstance(since, datetime.date):
            raise TypeError("The 'since' parameter must be a 'date' type")

        if not isinstance(until, datetime.date):
            raise TypeError("The 'until' parameter must be a 'date' type")

        if since > until:
            raise ValueError("'since' can't be more than 'until'")

        self.since = since
        self.until = until

    @property
    def delta_years(self):
        """
        Returns delta in years.
        """
        return self.until.year - self.since.year

    @property
    def delta_days(self):
        """
        Returns delta in days.
        """
        return (self.until - self.since).days

    def __iter__(self):
        """
        Iterate over time periods by day.
        """
        curren_date = self.since
        while curren_date <= self.until:
            yield curren_date
            curren_date += datetime.timedelta(days=1)


class RangePartitioning:
    """
    Range partitioning maps data to partitions based on ranges of values
    of the partitioning index name that you establish for each partition.
    It is the most common type of partitioning and is often used with dates.

    For an index with a date prefix as the partitioning key, the January-2018
    partition would has name with partitioning prefix from '2018-01-01'
    to '2018-01-31'.
    """

    def __init__(self,
                 frequency=DAY,
                 formatter=None,
                 escape="*",
                 now_func=None):
        """
        Initialize range partitioning instance.

        :param int frequency: Index partitioning frequency
        :param DateFormatter formatter: Formatter instance
        :param str escape: Special character which will be replaced on a date
        :param callable now_func: Get now date function
        """

        if formatter is None:
            self._formatter = BigEndianDateFormatter()
        else:
            if not isinstance(formatter, DateFormatter):
                raise TypeError(
                    "The 'formatter' must be an instance of 'DateFormatter'"
                )
            self._formatter = formatter

        self._frequency = frequency
        self._escape = escape
        self._now_func = now_func or self.__get_now_date

    def __get_now_date(self):
        return datetime.datetime.now().date()

    def partition(self, pattern, tm):
        """
        This method create and returns Elasticsearch partitioned indexes
        by date frequency.

        :param str pattern: Index name with a special character
        :param TimeWindow tm: Time window instance
        :rtype: list
        """

        def fmt_index(fmt_date):
            # Helper function for better index formatting
            return pattern.replace(self._escape, fmt_date)

        indexes = []
        for year, iterable in _group_by_year(tm):
            # Frequency by year
            if self._frequency == YEAR:
                fmt_date = self._formatter.fmt_year(year)
                indexes.append(pattern.replace(self._escape, fmt_date))
                continue

            # Temporary list for months and days indexes
            _indexes = []
            month_count = 0
            for month, iterable in _group_by_month(iterable):
                # Frequency by day
                if self._frequency > MONTH:
                    # Load all month days to a memory
                    days = list(iterable)
                    if len(days) != monthrange(year, month)[1]:
                        for date in days:
                            fmt_date = self._formatter.fmt_day(
                                date.year, date.month, date.day
                            )
                            _indexes.append(fmt_index(fmt_date))
                    else:
                        # Increase month counter and append formatted date
                        # to the temporary indexes list
                        month_count += 1
                        fmt_date = self._formatter.fmt_month(
                            year, month, wildcard=True
                        )
                        _indexes.append(fmt_index(fmt_date))

                # Frequency by month
                else:
                    month_count += 1
                    fmt_date = self._formatter.fmt_month(year, month)
                    _indexes.append(fmt_index(fmt_date))

            if month_count == _MAXMONTH:
                # Append formatted year with wildcard character if month
                # counter is full
                fmt_date = self._formatter.fmt_year(year,  wildcard=True)
                indexes.append(fmt_index(fmt_date))
            else:
                # Append temporary indexes to the main index list
                indexes.extend(_indexes)

        return indexes

    def __call__(self, pattern, since=None, until=None):
        """
        Returns Elasticsearch partitioned indexes by date range.

        :param str pattern: Index name with a special character
        :param datetime since: Since partitioning date
        :param datetime until: Until partitioning date
        """
        if self._escape not in pattern:
            raise ValueError("Index pattern '%s' doesn't contain a special "
                             "character '%s'" % (pattern, self._escape))

        if not since and not until:
            raise ValueError("You should use 'since' or 'until' for "
                             "searching by partitioning index")

        if not since:
            tm = TimeWindow(until, self._now_func())
            if self._frequency == YEAR and tm.delta_years < 1:
                indexes = [pattern]
            else:
                indexes = self.partition(_exclude(pattern), tm)
                indexes.append(pattern)
        else:
            tm = TimeWindow(since, until or self._now_func())
            indexes = self.partition(pattern, tm)

        return indexes


# Default Elasticsearch partitioning by day with big-endian date formatter
partition = RangePartitioning()
